==> kmuddy-0.7.1.tar.gz: kmuddy-0.7.1/kmuddy/dialogs/dlgtimers.cpp <==
/***************************************************************************
                          dlgtimers.cpp  -  description
                             -------------------
    begin                : Pi maj 9 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgtimers.h"
#include "../dialogs/dlgedittimer.h"
#include "../ctimer.h"
#include "../ctimerlist.h"

#include <qlabel.h>
#include <qlayout.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <kapp.h>
#include <kmessagebox.h>

dlgTimers::dlgTimers (cTimerList *tlist, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Timers"), Ok, Ok, true)
{
  tl = tlist;
  tl->setDialog (this);
  
  createDialog ();
}

dlgTimers::~dlgTimers ()
{
  tl->setDialog (0);
}

void dlgTimers::createDialog ()
{
  //make dialog size 70% of main window size, or 350x250, whichever is bigger
  QSize wsize = (KApplication::kApplication())->mainWidget()->frameSize();
  int dlgx = wsize.width() * 7 / 10;
  int dlgy = wsize.height() * 7 / 10;
  if (dlgx < 350) dlgx = 350;
  if (dlgy < 250) dlgy = 250;
  
  //initial dialog size
  setInitialSize (QSize (dlgx, dlgy));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 2, 2);

  setMainWidget (page);

  //change button text
  setButtonOKText (i18n ("Done"));

  //create widgets
  QLabel *label = new QLabel (i18n ("&List of timers"), page);
  box = new KListBox (page);
  label->setBuddy (box);
  QFrame *buttons = new QFrame (page);
  QVBoxLayout *buttonslayout = new QVBoxLayout (buttons);

  btadd = new KPushButton (i18n ("&Add..."), buttons);
  btmodify = new KPushButton (i18n ("&Modify..."), buttons);
  btdelete = new KPushButton (i18n ("&Delete"), buttons);
  btactivate = new KPushButton (i18n ("A&ctivate"), buttons);
  btdeactivate = new KPushButton (i18n ("D&eactivate"), buttons);

  buttonslayout->setSpacing (5);
  buttonslayout->addWidget (btadd);
  buttonslayout->addWidget (btmodify);
  buttonslayout->addWidget (btdelete);
  buttonslayout->addStretch (1);
  buttonslayout->addWidget (btactivate);
  buttonslayout->addWidget (btdeactivate);
  buttonslayout->addStretch (10);

  layout->setColStretch (0, 10);
  layout->setRowStretch (1, 10);
  layout->setSpacing (5);

  //place out widgets
  layout->addWidget (label, 0, 0);
  layout->addWidget (box, 1, 0);
  layout->addWidget (buttons, 1, 1);

  //update the listbox
  updateMe ();

  //connect() the whole stuff!
  connect (btadd, SIGNAL (clicked ()), this, SLOT (add ()));
  connect (btmodify, SIGNAL (clicked ()), this, SLOT (modify ()));
  connect (btdelete, SIGNAL (clicked ()), this, SLOT (remove ()));
  connect (btactivate, SIGNAL (clicked ()), this, SLOT (activate ()));
  connect (btdeactivate, SIGNAL (clicked ()), this, SLOT (deactivate ()));
  connect (box, SIGNAL (doubleClicked (QListBoxItem *)), this, SLOT (modify ()));
}

void dlgTimers::add ()
{
  //we show Edit timer dialog
  cTimer *timer = new cTimer (tl->session());
  dlgEditTimer *edit = new dlgEditTimer (timer, this);

  //determine position for the new object
  int it = box->currentItem ();
  
  if (edit->showMe ())
  {
    if (it >= 0)
    {
      tl->reset ();
      for (int i = 0; i < it; i++)
        (*tl)++;
    }
    //item now contains new data, so just add it to the list!
    timer->setActive (true);
    if (it >= 0)
      tl->addAfterCurrent (timer);
    else
      tl->addToEnd (timer);
    updateMe ();
  }
  else
    delete timer;
  delete edit;

  if (it >= 0)
    it++;
  else
    it = tl->count () - 1;
  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgTimers::modify ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  tl->reset ();
  for (int i = 0; i < it; i++)
    (*tl)++;
  cTimer *timer = (cTimer *) **tl;
  dlgEditTimer *edit = new dlgEditTimer (timer, this);
  if (edit->showMe ())
    //if he pressed OK, field already contains new data -> we only have to
    //update the listbox...
    updateMe ();
  delete edit;

  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgTimers::remove ()
{
  //get that item
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough

  //ask: really?
  int n = KMessageBox::questionYesNoCancel (this,
        i18n ("Do you really want to delete this timer?"),
        i18n ("Delete timer"));
  if (n == KMessageBox::Yes)
  {
    //really!
    //current item MUST be found AFTER the message box invocation; otherwise
    //some trigger matching could modify that
    tl->reset ();
    for (int i = 0; i < it; i++)
      (*tl)++;
    tl->removeCurrent ();
    updateMe ();
  }
}

void dlgTimers::activate ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  tl->reset ();
  for (int i = 0; i < it; i++)
    (*tl)++;
  ((cTimer *) **tl)->setActive (true);
  updateMe ();
}

void dlgTimers::deactivate ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  tl->reset ();
  for (int i = 0; i < it; i++)
    (*tl)++;
  ((cTimer *) **tl)->setActive (false);
  updateMe ();
}

void dlgTimers::updateMe ()
{
  //remember old index
  int idx = box->currentItem ();

  box->clear ();
  for (tl->reset(); (*tl); (*tl)++)
  {
    cTimer *t = (cTimer *) **tl;
    QString name = t->singleShot() ? i18n ("In ") : i18n ("Every ");
    name += QString::number (t->interval ());
    name += i18n (" seconds: ") + t->command ();
    if (t->active())
      name += i18n (" (active)");
    box->insertItem (name);
  }

  //make one item active, trying to be intelligent...
  box->setCurrentItem ((idx == -1) ? 0 : idx);
  if ((box->currentItem () == -1) && (idx != -1))
    box->setCurrentItem (idx - 1);
  if (box->currentItem () == -1)
    box->setCurrentItem (0);
}

==> kmuddy-0.8.tar.gz: kmuddy-0.8/kmuddy/dialogs/dlgtimers.cpp <==
/***************************************************************************
                          dlgtimers.cpp  -  description
                             -------------------
    begin                : Pi maj 9 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgtimers.h"
#include "dlgedittimer.h"
#include "ctimer.h"
#include "ctimerlist.h"

#include <qlabel.h>
#include <qlayout.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <kapp.h>
#include <kmessagebox.h>

dlgTimers::dlgTimers (cTimerList *tlist, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Timers"), Ok, Ok, true)
{
  tl = tlist;
  tl->setDialog (this);
  
  createDialog ();
}

dlgTimers::~dlgTimers ()
{
  tl->setDialog (0);
}

void dlgTimers::createDialog ()
{
  //make dialog size 70% of main window size, or 350x250, whichever is bigger
  QSize wsize = (KApplication::kApplication())->mainWidget()->frameSize();
  int dlgx = wsize.width() * 7 / 10;
  int dlgy = wsize.height() * 7 / 10;
  if (dlgx < 350) dlgx = 350;
  if (dlgy < 250) dlgy = 250;
  
  //initial dialog size
  setInitialSize (QSize (dlgx, dlgy));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 2, 2);

  setMainWidget (page);

  //change button text
  setButtonOKText (i18n ("Done"));

  //create widgets
  QLabel *label = new QLabel (i18n ("&List of timers"), page);
  box = new KListBox (page);
  label->setBuddy (box);
  QFrame *buttons = new QFrame (page);
  QVBoxLayout *buttonslayout = new QVBoxLayout (buttons);

  btadd = new KPushButton (i18n ("&Add..."), buttons);
  btmodify = new KPushButton (i18n ("&Modify..."), buttons);
  btdelete = new KPushButton (i18n ("&Delete"), buttons);
  btactivate = new KPushButton (i18n ("A&ctivate"), buttons);
  btdeactivate = new KPushButton (i18n ("D&eactivate"), buttons);

  buttonslayout->setSpacing (5);
  buttonslayout->addWidget (btadd);
  buttonslayout->addWidget (btmodify);
  buttonslayout->addWidget (btdelete);
  buttonslayout->addStretch (1);
  buttonslayout->addWidget (btactivate);
  buttonslayout->addWidget (btdeactivate);
  buttonslayout->addStretch (10);

  layout->setColStretch (0, 10);
  layout->setRowStretch (1, 10);
  layout->setSpacing (5);

  //place out widgets
  layout->addWidget (label, 0, 0);
  layout->addWidget (box, 1, 0);
  layout->addWidget (buttons, 1, 1);

  //update the listbox
  updateMe ();

  //connect() the whole stuff!
  connect (btadd, SIGNAL (clicked ()), this, SLOT (add ()));
  connect (btmodify, SIGNAL (clicked ()), this, SLOT (modify ()));
  connect (btdelete, SIGNAL (clicked ()), this, SLOT (remove ()));
  connect (btactivate, SIGNAL (clicked ()), this, SLOT (activate ()));
  connect (btdeactivate, SIGNAL (clicked ()), this, SLOT (deactivate ()));
  connect (box, SIGNAL (doubleClicked (QListBoxItem *)), this, SLOT (modify ()));
}

void dlgTimers::add ()
{
  //we show Edit timer dialog
  cTimer *timer = new cTimer (tl->sess());
  dlgEditTimer *edit = new dlgEditTimer (timer, this);

  //determine position for the new object
  int it = box->currentItem ();
  
  if (edit->showMe ())
  {
    if (it >= 0)
    {
      tl->reset ();
      for (int i = 0; i < it; i++)
        (*tl)++;
    }
    //item now contains new data, so just add it to the list!
    timer->setActive (true);
    if (it >= 0)
      tl->addAfterCurrent (timer);
    else
      tl->addToEnd (timer);
    updateMe ();
  }
  else
    delete timer;
  delete edit;

  if (it >= 0)
    it++;
  else
    it = tl->count () - 1;
  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgTimers::modify ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  tl->reset ();
  for (int i = 0; i < it; i++)
    (*tl)++;
  cTimer *timer = (cTimer *) **tl;
  dlgEditTimer *edit = new dlgEditTimer (timer, this);
  if (edit->showMe ())
    //if he pressed OK, field already contains new data -> we only have to
    //update the listbox...
    updateMe ();
  delete edit;

  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgTimers::remove ()
{
  //get that item
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough

  //ask: really?
  int n = KMessageBox::questionYesNoCancel (this,
        i18n ("Do you really want to delete this timer?"),
        i18n ("Delete timer"));
  if (n == KMessageBox::Yes)
  {
    //really!
    //current item MUST be found AFTER the message box invocation; otherwise
    //some trigger matching could modify that
    tl->reset ();
    for (int i = 0; i < it; i++)
      (*tl)++;
    tl->removeCurrent ();
    updateMe ();
  }
}

void dlgTimers::activate ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  tl->reset ();
  for (int i = 0; i < it; i++)
    (*tl)++;
  ((cTimer *) **tl)->setActive (true);
  updateMe ();
}

void dlgTimers::deactivate ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  tl->reset ();
  for (int i = 0; i < it; i++)
    (*tl)++;
  ((cTimer *) **tl)->setActive (false);
  updateMe ();
}

void dlgTimers::updateMe ()
{
  //remember old index
  int idx = box->currentItem ();

  box->clear ();
  for (tl->reset(); (*tl); (*tl)++)
  {
    cTimer *t = (cTimer *) **tl;
    QString name = t->singleShot() ? i18n ("In ") : i18n ("Every ");
    name += QString::number (t->interval ());
    name += i18n (" seconds: ") + t->command ();
    if (t->active())
      name += i18n (" (active)");
    box->insertItem (name);
  }

  //make one item active, trying to be intelligent...
  box->setCurrentItem ((idx == -1) ? 0 : idx);
  if ((box->currentItem () == -1) && (idx != -1))
    box->setCurrentItem (idx - 1);
  if (box->currentItem () == -1)
    box->setCurrentItem (0);
}
